//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	demo1.c
// @target	Atmel WinAVR C compiler
//
// This program demonstrates how to use the uM-FPU V3.1 floating point coprocessor
// connected to the Atmel AVR over an SPI interface. It takes temperature readings
// from a DS1620 digital thermometer, converts them to floating point and displays
// them in degrees Celsius and degrees Fahrenheit.
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	February 4, 2008
//	- added delay before data sample in read_DS1620
//	October 14, 2007
//	- original version
//=============================================================================

#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include "uart.h"
#include "fpu.h"

FILE uart_stream = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

#define ddr_set(bit) (_SFR_BYTE(DDRC) = _SFR_BYTE(DDRC) | _BV(bit))
#define ddr_clr(bit) (_SFR_BYTE(DDRC) = _SFR_BYTE(DDRC) & ~_BV(bit))
#define bit_set(bit) (_SFR_BYTE(PORTC) = _SFR_BYTE(PORTC) | _BV(bit))
#define bit_clr(bit) (_SFR_BYTE(PORTC) = _SFR_BYTE(PORTC) & ~_BV(bit))
#define bit_read(bit) (_SFR_BYTE(PINC) & _BV(bit))

#define DS_RST	0					// DS1620 reset
#define DS_CLK	1					// DS1620 clock
#define DS_DATA	2					// DS1620 data

//------------------ local prototypes -----------------------------------------

void init_DS1620(void);
void write_DS1620(char bval);
int read_DS1620(void);
void delay(int msec);

//-------------------- uM-FPU register definitions ----------------------------

#define DegC	1					// degrees Celsius
#define DegF	2					// degrees Fahrenheit
#define F1_8	3					// constant 1.8

//------------------ main -----------------------------------------------------

int main(void) {
    int rawTemp;

	// initialize the UART and set stdout
	uart_init();
	stdout = &uart_stream;

	printf("\r\n\r\nDemo1\r\n");

	// reset FPU and check synchronization
	if (fpu_reset() == SYNC_CHAR) {	
		print_version();
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		return(0);
	}

	// initizlize the DS1620 chip
	init_DS1620();

 	// store constant value (1.8)
	fpu_writeFloatReg(1.8, F1_8);

	// main sample loop
	while (1) {
		// get temperature reading from DS1620
		rawTemp = read_DS1620();

		// send to FPU and convert to floating point
		// divide by 2 to get degrees Celsius
		fpu_write3(SELECTA, DegC, LOADWORD);
		fpu_writeWord(rawTemp);
		fpu_write3(FSET0, FDIVI, 2);

		// degF = degC * 1.8 + 32
		fpu_write4(SELECTA, DegF, FSET, DegC);
		fpu_write4(FMUL, F1_8, FADDI, 32);

		// display degrees Celsius
		printf("\r\nDegrees C: ");
		fpu_write2(SELECTA, DegC);
		print_float(51);

		// display degrees Fahrenheit
		printf("\r\nDegrees F: ");
		fpu_write2(SELECTA, DegF);
		print_float(51);
		print_CRLF();

		// delay 2 seconds, then get the next reading
		delay(2000);
	}
}

//-------------------- init_DS1620 --------------------------------------------

void init_DS1620(void) {

	ddr_set(DS_RST);
	ddr_set(DS_CLK);
	ddr_set(DS_DATA);

	bit_clr(DS_RST);
	bit_set(DS_CLK);
	delay(100);

	bit_set(DS_RST);
	write_DS1620(0x0C);
	write_DS1620(0x02);
	bit_clr(DS_RST);
	delay(100);

	bit_set(DS_RST);
	write_DS1620(0xEE);
	bit_clr(DS_RST);
	delay(1000);
}

//-------------------- write_DS1620 -------------------------------------------

void write_DS1620(char bval) {
	int i;

	for (i = 0; i < 8; i++) {
		bit_clr(DS_CLK);
		if (bval & 0x01)
			bit_set(DS_DATA);
		else
			bit_clr(DS_DATA);
		bval >>= 1;
		bit_set(DS_CLK);
	}
}

//-------------------- read_DS1620 --------------------------------------------

int read_DS1620(void) {
	int tmp, i;

	bit_set(DS_RST);
	write_DS1620(0xAA);

	ddr_clr(DS_DATA);
	tmp = 0;
	for (i = 0; i < 9; i++) {
		bit_clr(DS_CLK);
		_delay_us(1);
		tmp >>= 1;
		if (bit_read(DS_DATA)) tmp |= 0x0100;
		bit_set(DS_CLK);
	}
	bit_clr(DS_CLK);
	ddr_set(DS_DATA);

	bit_clr(DS_RST);
	if (tmp & 0x0100) tmp |= 0xFF00;
	return tmp;
}

//-------------------- delay --------------------------------------------------

void delay(int msec) {
	int i;

	for (i = 0; i < msec; i += 10) {
		_delay_ms(10);
	}
}

